==> quidproquo-actionprocessor-awslambda-0.0.224.tgz: package/lib/esm/logic/dynamo/qpqDynamoOrm/buildDynamoQuery.js <==
import crypto from 'crypto';
import { KvsQueryOperationType, KvsLogicalOperatorType, } from 'quidproquo-core';
// TODO: Come up with a better way of generating item name / values
// probably a map ~ however this will most likely never fail.
const getHash = (name) => {
    const c = crypto.createHash('md5').update(name).digest('hex');
    return c;
};
export const getItemName = (name) => {
    return `#${getHash(name)}`;
};
export const getValueName = (value) => {
    return `:${getHash(`${typeof value}-${JSON.stringify(value)}`)}`;
};
const buildDynamoQueryExpressionBetween = (query) => {
    if (!query.key || !query.valueB || !query.valueB) {
        throw new Error(`Invalid query condition ${KvsQueryOperationType.Between}`);
    }
    return `${getItemName(query.key)} BETWEEN ${getValueName(query.valueA)} AND ${getValueName(query.valueB)}`;
};
const buildDynamoQueryExpressionEqual = (query) => {
    return `${getItemName(query.key)} = ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionNotEqual = (query) => {
    return `${getItemName(query.key)} <> ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionLessThan = (query) => {
    return `${getItemName(query.key)} < ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionLessThanOrEqual = (query) => {
    return `${getItemName(query.key)} <= ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionGreaterThan = (query) => {
    return `${getItemName(query.key)} > ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionGreaterThanOrEqual = (query) => {
    return `${getItemName(query.key)} >= ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionIn = (query) => {
    return `${getItemName(query.key)} IN (${query.valueA
        .map((v) => getValueName(v))
        .join(', ')})`;
};
const buildDynamoQueryExpressionExists = (query) => {
    return `attribute_exists(${getItemName(query.key)})`;
};
const buildDynamoQueryExpressionNotExists = (query) => {
    return `attribute_not_exists(${getItemName(query.key)})`;
};
const buildDynamoQueryExpressionBeginsWith = (query) => {
    return `begins_with(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionContains = (query) => {
    return `contains(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionNotContains = (query) => {
    return `NOT contains(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionOr = (query) => {
    return query.conditions.map((c) => `(${buildDynamoQueryExpressionRoot(c)})`).join(' OR ');
};
const buildDynamoQueryExpressionAnd = (query) => {
    return query.conditions.map((c) => `(${buildDynamoQueryExpressionRoot(c)})`).join(' AND ');
};
export const isKvsQueryCondition = (query) => {
    return 'key' in query && 'operation' in query;
};
export const isKvsLogicalOperator = (query) => {
    return 'conditions' in query && 'operation' in query;
};
const buildDynamoQueryExpressionRoot = (query) => {
    if (isKvsQueryCondition(query)) {
        switch (query.operation) {
            case KvsQueryOperationType.Equal:
                return buildDynamoQueryExpressionEqual(query);
            case KvsQueryOperationType.NotEqual:
                return buildDynamoQueryExpressionNotEqual(query);
            case KvsQueryOperationType.LessThan:
                return buildDynamoQueryExpressionLessThan(query);
            case KvsQueryOperationType.LessThanOrEqual:
                return buildDynamoQueryExpressionLessThanOrEqual(query);
            case KvsQueryOperationType.GreaterThan:
                return buildDynamoQueryExpressionGreaterThan(query);
            case KvsQueryOperationType.GreaterThanOrEqual:
                return buildDynamoQueryExpressionGreaterThanOrEqual(query);
            case KvsQueryOperationType.Between:
                return buildDynamoQueryExpressionBetween(query);
            case KvsQueryOperationType.In:
                return buildDynamoQueryExpressionIn(query);
            case KvsQueryOperationType.Exists:
                return buildDynamoQueryExpressionExists(query);
            case KvsQueryOperationType.NotExists:
                return buildDynamoQueryExpressionNotExists(query);
            case KvsQueryOperationType.BeginsWith:
                return buildDynamoQueryExpressionBeginsWith(query);
            case KvsQueryOperationType.Contains:
                return buildDynamoQueryExpressionContains(query);
            case KvsQueryOperationType.NotContains:
                return buildDynamoQueryExpressionNotContains(query);
        }
    }
    else if (isKvsLogicalOperator(query)) {
        switch (query.operation) {
            case KvsLogicalOperatorType.And:
                return buildDynamoQueryExpressionAnd(query);
            case KvsLogicalOperatorType.Or:
                return buildDynamoQueryExpressionOr(query);
        }
    }
    throw new Error(`Invalid query operation: ${JSON.stringify(query)}`);
};
export const buildAttributeValue = (value) => {
    const valueType = typeof value;
    switch (valueType) {
        case 'string':
            return { S: value };
        case 'number':
            return { N: value.toString() };
        case 'boolean':
            return { BOOL: value };
        case 'object':
            if (Array.isArray(value)) {
                return {
                    L: value.map((item) => buildAttributeValue(item)),
                };
            }
            else if (value !== null) {
                const objectKeyValuePairs = Object.entries(value)
                    .filter(([k, v]) => v !== undefined)
                    .map(([k, v]) => {
                    return [k, buildAttributeValue(v)];
                });
                return {
                    M: Object.fromEntries(objectKeyValuePairs),
                };
            }
            else {
                return { NULL: true };
            }
        default:
            throw new Error(`Unsupported data type in kvs expression: ${valueType}`);
    }
};
export const buildDynamoQueryExpression = (query) => {
    if (!query) {
        return undefined;
    }
    return buildDynamoQueryExpressionRoot(query);
};
export const buildExpressionAttributeValues = (queries) => {
    const values = {};
    const traverse = (query) => {
        if (isKvsQueryCondition(query)) {
            if (query.valueA !== undefined) {
                const valueNameA = getValueName(query.valueA);
                values[valueNameA] = buildAttributeValue(query.valueA);
            }
            if (query.valueB !== undefined) {
                const valueNameB = getValueName(query.valueB);
                values[valueNameB] = buildAttributeValue(query.valueB);
            }
        }
        else if (isKvsLogicalOperator(query)) {
            for (const condition of query.conditions) {
                traverse(condition);
            }
        }
    };
    queries.filter((q) => !!q).forEach((q) => traverse(q));
    return Object.keys(values).length > 0 ? values : undefined;
};
export const buildExpressionAttributeNames = (queries) => {
    const names = {};
    const traverse = (query) => {
        if (isKvsQueryCondition(query)) {
            const itemName = getItemName(query.key);
            names[itemName] = query.key;
        }
        else if (isKvsLogicalOperator(query)) {
            for (const condition of query.conditions) {
                traverse(condition);
            }
        }
    };
    queries.filter((q) => !!q).forEach((q) => traverse(q));
    return Object.keys(names).length > 0 ? names : undefined;
};

==> quidproquo-actionprocessor-awslambda-0.0.242.tgz: package/lib/esm/logic/dynamo/qpqDynamoOrm/buildDynamoQuery.js <==
import { KvsLogicalOperatorType, KvsQueryOperationType, } from 'quidproquo-core';
import crypto from 'crypto';
// TODO: Come up with a better way of generating item name / values
// probably a map ~ however this will most likely never fail.
const getHash = (name) => {
    const c = crypto.createHash('md5').update(name).digest('hex');
    return c;
};
export const getItemName = (name) => {
    return `#${getHash(name)}`;
};
export const getValueName = (value) => {
    return `:${getHash(`${typeof value}-${JSON.stringify(value)}`)}`;
};
const buildDynamoQueryExpressionBetween = (query) => {
    if (!query.key || !query.valueB || !query.valueB) {
        throw new Error(`Invalid query condition ${KvsQueryOperationType.Between}`);
    }
    return `${getItemName(query.key)} BETWEEN ${getValueName(query.valueA)} AND ${getValueName(query.valueB)}`;
};
const buildDynamoQueryExpressionEqual = (query) => {
    return `${getItemName(query.key)} = ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionNotEqual = (query) => {
    return `${getItemName(query.key)} <> ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionLessThan = (query) => {
    return `${getItemName(query.key)} < ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionLessThanOrEqual = (query) => {
    return `${getItemName(query.key)} <= ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionGreaterThan = (query) => {
    return `${getItemName(query.key)} > ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionGreaterThanOrEqual = (query) => {
    return `${getItemName(query.key)} >= ${getValueName(query.valueA)}`;
};
const buildDynamoQueryExpressionIn = (query) => {
    return `${getItemName(query.key)} IN (${query.valueA.map((v) => getValueName(v)).join(', ')})`;
};
const buildDynamoQueryExpressionExists = (query) => {
    return `attribute_exists(${getItemName(query.key)})`;
};
const buildDynamoQueryExpressionNotExists = (query) => {
    return `attribute_not_exists(${getItemName(query.key)})`;
};
const buildDynamoQueryExpressionBeginsWith = (query) => {
    return `begins_with(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionContains = (query) => {
    return `contains(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionNotContains = (query) => {
    return `NOT contains(${getItemName(query.key)}, ${getValueName(query.valueA)})`;
};
const buildDynamoQueryExpressionOr = (query) => {
    return query.conditions.map((c) => `(${buildDynamoQueryExpressionRoot(c)})`).join(' OR ');
};
const buildDynamoQueryExpressionAnd = (query) => {
    return query.conditions.map((c) => `(${buildDynamoQueryExpressionRoot(c)})`).join(' AND ');
};
export const isKvsQueryCondition = (query) => {
    return 'key' in query && 'operation' in query;
};
export const isKvsLogicalOperator = (query) => {
    return 'conditions' in query && 'operation' in query;
};
const buildDynamoQueryExpressionRoot = (query) => {
    if (isKvsQueryCondition(query)) {
        switch (query.operation) {
            case KvsQueryOperationType.Equal:
                return buildDynamoQueryExpressionEqual(query);
            case KvsQueryOperationType.NotEqual:
                return buildDynamoQueryExpressionNotEqual(query);
            case KvsQueryOperationType.LessThan:
                return buildDynamoQueryExpressionLessThan(query);
            case KvsQueryOperationType.LessThanOrEqual:
                return buildDynamoQueryExpressionLessThanOrEqual(query);
            case KvsQueryOperationType.GreaterThan:
                return buildDynamoQueryExpressionGreaterThan(query);
            case KvsQueryOperationType.GreaterThanOrEqual:
                return buildDynamoQueryExpressionGreaterThanOrEqual(query);
            case KvsQueryOperationType.Between:
                return buildDynamoQueryExpressionBetween(query);
            case KvsQueryOperationType.In:
                return buildDynamoQueryExpressionIn(query);
            case KvsQueryOperationType.Exists:
                return buildDynamoQueryExpressionExists(query);
            case KvsQueryOperationType.NotExists:
                return buildDynamoQueryExpressionNotExists(query);
            case KvsQueryOperationType.BeginsWith:
                return buildDynamoQueryExpressionBeginsWith(query);
            case KvsQueryOperationType.Contains:
                return buildDynamoQueryExpressionContains(query);
            case KvsQueryOperationType.NotContains:
                return buildDynamoQueryExpressionNotContains(query);
        }
    }
    else if (isKvsLogicalOperator(query)) {
        switch (query.operation) {
            case KvsLogicalOperatorType.And:
                return buildDynamoQueryExpressionAnd(query);
            case KvsLogicalOperatorType.Or:
                return buildDynamoQueryExpressionOr(query);
        }
    }
    throw new Error(`Invalid query operation: ${JSON.stringify(query)}`);
};
export const buildAttributeValue = (value) => {
    const valueType = typeof value;
    switch (valueType) {
        case 'string':
            return { S: value };
        case 'number':
            return { N: value.toString() };
        case 'boolean':
            return { BOOL: value };
        case 'object':
            if (Array.isArray(value)) {
                return {
                    L: value.map((item) => buildAttributeValue(item)),
                };
            }
            else if (value !== null) {
                const objectKeyValuePairs = Object.entries(value)
                    .filter(([k, v]) => v !== undefined)
                    .map(([k, v]) => {
                    return [k, buildAttributeValue(v)];
                });
                return {
                    M: Object.fromEntries(objectKeyValuePairs),
                };
            }
            else {
                return { NULL: true };
            }
        default:
            throw new Error(`Unsupported data type in kvs expression: ${valueType}`);
    }
};
export const buildDynamoQueryExpression = (query) => {
    if (!query) {
        return undefined;
    }
    return buildDynamoQueryExpressionRoot(query);
};
export const buildExpressionAttributeValues = (queries) => {
    const values = {};
    const traverse = (query) => {
        if (isKvsQueryCondition(query)) {
            if (query.valueA !== undefined) {
                const valueNameA = getValueName(query.valueA);
                values[valueNameA] = buildAttributeValue(query.valueA);
            }
            if (query.valueB !== undefined) {
                const valueNameB = getValueName(query.valueB);
                values[valueNameB] = buildAttributeValue(query.valueB);
            }
        }
        else if (isKvsLogicalOperator(query)) {
            for (const condition of query.conditions) {
                traverse(condition);
            }
        }
    };
    queries.filter((q) => !!q).forEach((q) => traverse(q));
    return Object.keys(values).length > 0 ? values : undefined;
};
export const buildExpressionAttributeNames = (queries) => {
    const names = {};
    const traverse = (query) => {
        if (isKvsQueryCondition(query)) {
            const itemName = getItemName(query.key);
            names[itemName] = query.key;
        }
        else if (isKvsLogicalOperator(query)) {
            for (const condition of query.conditions) {
                traverse(condition);
            }
        }
    };
    queries.filter((q) => !!q).forEach((q) => traverse(q));
    return Object.keys(names).length > 0 ? names : undefined;
};
